Package: xpose.nlmixr2
Type: Package
Title: Graphical Diagnostics for Pharmacometric Models: Extension to
        'nlmixr2'
Version: 0.4.2
Authors@R: c(person('Justin', 'Wilkins', email = 'justin.wilkins@occams.com',
                  role = c('aut', 'cre', 'cph')),
          person('Matthew','Fidler',
      role = c('aut','cph'), email = 'matthew.fidler@gmail.com'),
            person('Benjamin', 'Guiastrennec',
             role = c('aut'),
             email = 'guiastrennec@gmail.com'),
      person('Andrew C.', 'Hooker',
             role = c('aut'),
             email = 'andrew.hooker@farmbio.uu.se'),
      person('Anna', 'Olofsson', role = c('aut', 'cph')),
      person('Sebastian', 'Ueckert',
             role = c('aut'),
             email = 'sebastian.ueckert@farmbio.uu.se'),
      person('Ron', 'Keizer',
             role = 'aut',
             email = 'ronkeizer@gmail.com'),
      person('Kajsa', 'Harling', role = 'ctb'),
      person('Mike K.', 'Smith', role = 'ctb'),
      person('Elodie', 'Plan', role = 'ctb'),
      person('Mats O.', 'Karlsson',
             role = c('aut', 'cph'),
             email = 'mats.karlsson@farmbio.uu.se'),
      person('Pharmetheus', role = 'ctb'),
      person('Pfizer', role = 'ctb'),
      person('Occams', role = 'ctb'),
      person('Novartis', role = 'ctb'))
Maintainer: Justin Wilkins <justin.wilkins@occams.com>
Description: Extension to 'xpose' to support 'nlmixr2'. Provides functions to import 'nlmixr2' fit data into an 'xpose' data object, allowing the use of 'xpose' for 'nlmixr2' model diagnostics.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.2), xpose (>= 0.4.2)
Imports: ggplot2 (>= 2.2.1), dplyr (>= 0.7.4), tibble (>= 2.0.0),
        stringr (>= 1.2.0), tidyr (>= 0.7.2), magrittr (>= 1.5),
        methods (>= 3.4.1), vpc (>= 1.0.2), crayon, rlang, nlmixr2est
Suggests: readr, nlmixr2, nlmixr2data, testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-09 06:38:45 UTC; justin
Author: Justin Wilkins [aut, cre, cph],
  Matthew Fidler [aut, cph],
  Benjamin Guiastrennec [aut],
  Andrew C. Hooker [aut],
  Anna Olofsson [aut, cph],
  Sebastian Ueckert [aut],
  Ron Keizer [aut],
  Kajsa Harling [ctb],
  Mike K. Smith [ctb],
  Elodie Plan [ctb],
  Mats O. Karlsson [aut, cph],
  Pharmetheus [ctb],
  Pfizer [ctb],
  Occams [ctb],
  Novartis [ctb]
Repository: CRAN
Date/Publication: 2026-05-09 14:40:02 UTC
Built: R 4.6.0; ; 2026-05-09 18:57:49 UTC; unix

# Generated by roxygen2: do not edit by hand

export(nlmixr2_vpc_theme)
export(theme_xp_nlmixr2)
export(xpose_data_nlmixr)
export(xpose_data_nlmixr2)
importFrom(dplyr,arrange_at)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,one_of)
importFrom(dplyr,select)
importFrom(dplyr,tibble)
importFrom(ggplot2,label_both)
importFrom(ggplot2,labeller)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(nlmixr2est,nlmixr2)
importFrom(rlang,syms)
importFrom(stats,coef)
importFrom(stats,rnorm)
importFrom(stringr,str_detect)
importFrom(tibble,as_tibble)
importFrom(tidyr,complete)
importFrom(vpc,new_vpc_theme)
importFrom(xpose,theme_readable)
importFrom(xpose,theme_xp_default)

# xpose.nlmixr2

<!-- badges: start -->
[![CRAN checks](https://cranchecks.info/badges/summary/xpose.nlmixr2)](https://cran.r-project.org/web/checks/check_results_xpose.nlmixr2.html)
[![CRAN total downloads](https://cranlogs.r-pkg.org/badges/grand-total/xpose.nlmixr2)](https://cran.r-project.org/package=xpose.nlmixr2)
<!-- badges: end -->

The goal of `xpose.nlmixr2` is to provide an extension to `xpose` to
support `nlmixr2`. The package provides functions to import `nlmixr2`
fit data into an `xpose` data object, allowing the use of `xpose` for
`nlmixr2` model diagnostics.

The excellent `xpose` package can be found on [CRAN](https://cran.r-project.org/package=xpose) or on 
[GitHub](https://github.com/UUPharmacometrics/xpose), and is a
prerequisite for using `xpose.nlmixr2`.

## Installation

You can install the released version of `xpose.nlmixr2` from [CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("xpose.nlmixr2")
```

Alternatively, the development version can be installed from [GitHub](https://github.com/nlmixr2/xpose.nlmixr2):

``` r
devtools::install_github("nlmixr2/xpose.nlmixr2")
```

#' Default VPC theme for 'xpose.nlmixr2'
#'
#' @description Default VPC theme for 'xpose.nlmixr2'.
#'
#' @return A list with 'vpc' theme specifiers.
#'
#' @importFrom vpc new_vpc_theme
#' @importFrom xpose theme_xp_default
#'
#' @export
nlmixr2_vpc_theme <- vpc::new_vpc_theme(list(
  obs_color = "#1F4E79",
  obs_size = 1,
  obs_median_color = "#1F4E79",
  obs_median_linetype = "solid",
  obs_median_size = 1,
  obs_alpha = .7,
  obs_shape = 1,
  obs_ci_color = "#1F4E79",
  obs_ci_linetype = "dashed",
  obs_ci_size = .5,
  sim_pi_fill = "#C00000",
  sim_pi_alpha = 0.15,
  sim_pi_color = "#1F4E79",
  sim_pi_linetype = 'dotted',
  sim_pi_size = 1,
  sim_median_fill = "#C00000",
  sim_median_alpha = 0.15,
  sim_median_color = "#1F4E79",
  sim_median_linetype = "dashed",
  sim_median_size = 1,
  bin_separators_color = "#000000"
))

#' Import nlmixr2 output into xpose object
#'
#' @description Convert 'nlmixr2' model output into an 'xpose' database.
#'
#' @param obj nlmixr2 fit object to be evaluated.
#' @param pred Name of the population prediction variable to use for
#'     plotting. If unspecified, it will choose either "NPDE",
#'     "CWRES", and "RES" (in that order) if the column exists in the
#'     data.
#' @param wres Name of the weighted residual variable to use for
#'     plotting. If unspecified, it will choose either "NPDE",
#'     "CWRES", and "RES" (in that order) if the column exists in the
#'     data.
#' @param gg_theme A ggplot2 theme object.
#' @param xp_theme An xpose theme or vector of modifications to the
#'     xpose theme (eg. \code{c(point_color = 'red', line_linetype =
#'     'dashed')}).
#' @param quiet Logical, if \code{FALSE} messages are printed to the
#'     console.
#' @param skip Character vector be used to skip the import/generation
#'     of: 'data', 'files', 'summary' or any combination of the three.
#' @param ... Additional arguments to be passed to the
#'     \code{\link[readr]{read_delim}} functions.
#'
#' @return An \code{\link[xpose]{xpose_data}} object suitable for use in 'xpose'.
#'
#' @importFrom nlmixr2est nlmixr2
#' @importFrom dplyr group_by mutate tibble case_when
#' @importFrom tibble as_tibble
#' @importFrom stringr str_detect
#' @importFrom xpose theme_readable theme_xp_default
#' @importFrom stats coef rnorm
#'
#' @examples
#' \dontrun{
#' library(nlmixr2)
#'
#' one.cmt <- function() {
#'   ini({
#'     ## You may label each parameter with a comment
#'     tka <- 0.45 # Ka
#'     tcl <- log(c(0, 2.7, 100)) # Log Cl
#'    ## This works with interactive models
#'     ## You may also label the preceding line with label("label text")
#'     tv <- 3.45; label("log V")
#'     ## the label("Label name") works with all models
#'     eta.ka ~ 0.6
#'     eta.cl ~ 0.3
#'     eta.v ~ 0.1
#'     add.sd <- 0.7
#'   })
#'   model({
#'     ka <- exp(tka + eta.ka)
#'     cl <- exp(tcl + eta.cl)
#'     v <- exp(tv + eta.v)
#'     linCmt() ~ add(add.sd)
#'   })
#' }
#'
#' theo_sd_fit <- nlmixr2(one.cmt, theo_sd, "focei", control=foceiControl(print=0))
#'
#' xpdb <- xpose_data_nlmixr2(obj = theo_sd_fit)
#'}
#'
#' @export
xpose_data_nlmixr2 <- function(
  obj = NULL,
  pred = NULL, #"CPRED",
  wres = NULL, #"CWRES",
  gg_theme = theme_readable,
  xp_theme = theme_xp_default(),
  quiet,
  skip = NULL,
  ...
) {
  runname <- deparse(substitute(obj))

  . = NULL
  ID = NULL
  RES = NULL
  DV = NULL
  PRED = NULL
  msg = NULL

  get_wres <- function(res, dv, pred) {
    suppressWarnings(res / (sqrt(stats::cov(dv, pred))))
  }

  if (is.null(obj)) {
    stop('Argument `obj` required.', call. = FALSE)
  }

  if (missing(quiet)) {
    quiet <- !interactive()
  }

  if (!inherits(obj, "nlmixr2FitData")) {
    stop("Input object needs to be an nlmixr2 fit.")
  }
  mtype <- obj$est
  software <- "nlmixr2"
  if (is.null(wres)) {
    if (any(names(obj) == "CWRES")) {
      wres <- "CWRES"
    } else if (any(names(obj) == "NPDE")) {
      wres <- "NPDE"
    } else if (any(names(obj) == "RES")) {
      wres <- "RES"
      obj <- nlmixr2est::addCwres(obj)
      if (any(names(obj) == "CWRES")) {
        wres <- "CWRES"
        warning(sprintf(
          "Added CWRES to fit (using %s%s)...",
          crayon::blue("nlmixr2est::"),
          crayon::yellow("addCwres")
        ))
      } else {
        warning(sprintf(
          "Using RES; Consider adding NPDE (%s%s) to fit.",
          crayon::blue("nlmixr2est::"),
          crayon::yellow("addNpde")
        ))
      }
    }
  }
  if (is.null(wres)) {
    stop(
      "Could not automatically determine the weighted residual column. ",
      "The fit does not contain CWRES, NPDE, or RES. ",
      "For models without IIV or using ll(), specify `pred` and `wres` manually ",
      "(e.g. pred = \"IPRED\", wres = \"IWRES\").",
      call. = FALSE
    )
  }

  if (is.null(pred)) {
    if (any(names(obj) == "EPRED") && wres == "NPDE") {
      pred <- "EPRED"
    } else if (any(names(obj) == "CPRED") && wres == "CWRES") {
      pred <- "CPRED"
    } else if (any(names(obj) == "PRED") && wres == "RES") {
      pred <- "PRED"
    } else if (any(names(obj) == "EPRED")) {
      pred <- "EPRED"
    } else if (any(names(obj) == "CPRED")) {
      pred <- "CPRED"
    } else if (any(names(obj) == "PRED")) {
      pred <- "PRED"
    }
  }

  if (is.null(pred)) {
    stop(
      "Could not automatically determine the population prediction column. ",
      "The fit does not contain EPRED, CPRED, or PRED. ",
      "Specify `pred` manually (e.g. pred = \"IPRED\").",
      call. = FALSE
    )
  }

  if (any("nlmixr2FitData" == class(obj))) {
    data <- as.data.frame(obj)
    data_a <- data %>%
      dplyr::group_by(ID)

    data_a <- tibble::as_tibble(data_a)
  }

  if (!(wres %in% names(data_a))) {
    stop(
      paste(wres, ' not found in nlmixr2 fit object.', sep = ""),
      call. = FALSE
    )
  }

  if (!(pred %in% names(data_a))) {
    stop(
      paste(pred, ' not found in nlmixr2 fit object.', sep = ""),
      call. = FALSE
    )
  }

  if (!inherits(obj, "nlmixr2FitData")) {
    stop("needs to be a nlmixr2 fit object")
  }
  data_a <- obj$dataMergeInner
  names(data_a) <- toupper(names(data_a))
  data_a <- data_a[, !duplicated(names(data_a))]
  full.dat <- data_a

  # check for ETAs
  # if(!any(stringr::str_detect(names(data_a), 'ETA\\d+|ET\\d+|eta.*'))) {
  #   data_a <- merge(data_a, obj$eta)
  # }
  if (!all(names(diag(obj$omega)) %in% names(data_a))) {
    data_a <- merge(data_a, obj$eta)
  }

  if (!("EVID" %in% names(data_a))) {
    data_a$EVID <- 0
  }
  #print(summary(data_a))

  data <- NULL
  data_ind <- data_a %>%
    colnames() %>%
    dplyr::tibble(
      table = 'nlmixr2',
      col = .,
      type = NA_character_,
      label = NA_character_, # Feature to be added in future release
      units = NA_character_
    ) %>% # Feature to be added in future release
    dplyr::mutate(
      type = dplyr::case_when(
        .$col == 'ID' ~ 'id',
        .$col == 'DV' ~ 'dv',
        .$col == 'TIME' ~ 'idv',
        .$col == 'OCC' ~ 'occ',
        .$col == 'DVID' ~ 'dvid',
        .$col == 'AMT' ~ 'amt',
        .$col == 'MDV' ~ 'mdv',
        .$col == 'EVID' ~ 'evid',
        .$col == 'IPRED' ~ 'ipred',
        .$col == pred ~ 'pred',
        .$col %in%
          c(
            'RES',
            'WRES',
            'CWRES',
            'IWRES',
            'EWRES',
            'NPDE',
            'IRES',
            'CRES'
          ) ~ 'res',
        .$col %in% c('WT', 'AGE', 'HT', 'BMI', 'LBM') ~ 'contcov',
        .$col %in% c('SEX', 'RACE') ~ 'catcov',
        .$col %in%
          c(
            'CL',
            'V',
            'V1',
            'V2',
            'V3',
            'Q',
            'Q2',
            'Q3',
            'KA',
            'K12',
            'K21',
            'K',
            'K13',
            'K31',
            'K23',
            'K32',
            'K24',
            'K42',
            'K34',
            'K43',
            'cl',
            'v',
            'v1',
            'v2',
            'v3',
            'q',
            'q2',
            'q3',
            'ka',
            'k12',
            'k21',
            'k',
            'k13',
            'k31',
            'k23',
            'k32',
            'k24',
            'k42',
            'k34',
            'k43',
            'tcl',
            'tv',
            'tv1',
            'tv2',
            'tv3',
            'tq',
            'tq2',
            'tq3',
            'tka',
            'tk12',
            'tk21',
            'tk',
            'tk13',
            'tk31',
            'tk23',
            'tk32',
            'tk24',
            'tk42',
            'tk34',
            'tk43'
          ) ~ 'param',
        stringr::str_detect(.$col, 'ETA\\d+|ET\\d+|eta.*') ~ 'eta'
      )
    )

  data_ind$type[is.na(data_ind$type)] <- 'na'

  data <- list()
  data <- dplyr::tibble(
    problem = 1,
    simtab = F,
    index = list(data_ind),
    data = list(data_a),
    modified = F
  )

  # Generate model summary
  if ('summary' %in% skip) {
    msg('Skipping summary generation', quiet)
    summary <- NULL
  } else if (software == 'nlmixr2') {
    summary <- summarise_nlmixr2_model(
      obj,
      '',
      software,
      rounding = xp_theme$rounding,
      runname = runname
    )
  }

  # The weighted residuals are calculated by dividing the vector of each
  # individual's residuals (res_i) by the square root of the matrix of
  # covariances of that individual's data conditional on the population model:
  #
  #   WRES_i = RES_i / SQRT(COV(data_i | F_pop))
  #
  # This means that for each WRES calculated we include covariances between data
  # points of an individual.  If the correlations between some of these data
  # points are negative then the resulting WRES could also be negative, while
  # the RES could be positive.
  #
  # -Andy

  files <- NULL
  if (!is.null(obj$par.hist) && nrow(obj$par.hist) > 0) {
    tracedat <- tibble::as_tibble(as.data.frame(obj$par.hist))
    names(tracedat)[grep("iter", names(tracedat))] <-
      "ITERATION"

    files <- dplyr::tibble(
      name = deparse(substitute(obj)),
      extension = 'ext',
      problem = 1,
      subprob = 0,
      method = mtype,
      data = list(tracedat),
      modified = FALSE
    )
  }

  # Label themes
  attr(gg_theme, 'theme') <- as.character(substitute(gg_theme))
  attr(xp_theme, 'theme') <- as.character(substitute(xp_theme))

  # Output xpose_data
  list(
    code = obj$uif,
    summary = summary,
    data = data,
    files = files,
    gg_theme = gg_theme,
    xp_theme = xp_theme,
    options = list(dir = NULL, quiet = quiet, manual_import = NULL),
    software = 'nlmixr2'
  ) %>%
    structure(class = c('xpose_data', 'uneval'))
}

#' @rdname xpose_data_nlmixr2
#' @export
xpose_data_nlmixr <- xpose_data_nlmixr2

test_that("xpose_data object is valid", {
  one.cmt <- function() {
    ini({
      ## You may label each parameter with a comment
      tka <- 0.45 # Ka
      tcl <- log(c(0, 2.7, 100)) # Log Cl
      ## This works with interactive models
      ## You may also label the preceding line with label("label text")
      tv <- 3.45
      label("log V")
      ## the label("Label name") works with all models
      eta.ka ~ 0.6
      eta.cl ~ 0.3
      eta.v ~ 0.1
      add.sd <- 0.7
    })
    model({
      ka <- exp(tka + eta.ka)
      cl <- exp(tcl + eta.cl)
      v <- exp(tv + eta.v)
      linCmt() ~ add(add.sd)
    })
  }

  theo_sd_fit <- nlmixr2est::nlmixr2(
    one.cmt,
    nlmixr2data::theo_sd,
    "focei",
    control = nlmixr2est::foceiControl(print = 0)
  )

  xpdb_nlmixr2 <- xpose_data_nlmixr2(theo_sd_fit)

  # Basic checks
  expect_true(xpose::is.xpdb(xpdb_nlmixr2))

  expect_no_error(xpose::check_xpdb(xpdb_nlmixr2))

  # Summary checks
  typical_labels <- xpose::xpdb_ex_pk$summary$label
  expect_setequal(
    typical_labels,
    xpdb_nlmixr2$summary$label
  )

  # par.hist available for focei: files should be populated
  expect_false(is.null(xpdb_nlmixr2$files))
  expect_equal(xpdb_nlmixr2$files$method, "focei")
  expect_true("ITERATION" %in% names(xpdb_nlmixr2$files$data[[1]]))
})

test_that("FOCEi fit with eta fixed to zero (eta ~ 0) succeeds (#10)", {
  one.cmt.fixed.ka <- function() {
    ini({
      tka <- 0.45
      tcl <- log(2.7)
      tv <- 3.45
      eta.ka ~ 0
      eta.cl ~ 0.3
      eta.v ~ 0.1
      add.sd <- 0.7
    })
    model({
      ka <- exp(tka + eta.ka)
      cl <- exp(tcl + eta.cl)
      v <- exp(tv + eta.v)
      linCmt() ~ add(add.sd)
    })
  }

  fit_fixed_ka <- nlmixr2est::nlmixr2(
    one.cmt.fixed.ka,
    nlmixr2data::theo_sd,
    "focei",
    control = nlmixr2est::foceiControl(print = 0)
  )

  xpdb <- xpose_data_nlmixr2(fit_fixed_ka)
  expect_true(xpose::is.xpdb(xpdb))
})

test_that("no-IIV fit without pred/wres gives a descriptive error (#8)", {
  no_bsv <- function() {
    ini({
      tka <- 0.45
      tcl <- log(2.7)
      tv <- 3.45
      add.sd <- 0.7
    })
    model({
      ka <- exp(tka)
      cl <- exp(tcl)
      v <- exp(tv)
      linCmt() ~ add(add.sd)
    })
  }

  fit_no_bsv <- nlmixr2est::nlmixr2(
    no_bsv,
    nlmixr2data::theo_sd,
    "focei",
    control = nlmixr2est::foceiControl(print = 0)
  )

  expect_snapshot(error = TRUE, xpose_data_nlmixr2(fit_no_bsv))
})

test_that("no-IIV fit with manual pred/wres succeeds (#8)", {
  no_bsv <- function() {
    ini({
      tka <- 0.45
      tcl <- log(2.7)
      tv <- 3.45
      add.sd <- 0.7
    })
    model({
      ka <- exp(tka)
      cl <- exp(tcl)
      v <- exp(tv)
      linCmt() ~ add(add.sd)
    })
  }

  fit_no_bsv <- nlmixr2est::nlmixr2(
    no_bsv,
    nlmixr2data::theo_sd,
    "focei",
    control = nlmixr2est::foceiControl(print = 0)
  )

  xpdb <- xpose_data_nlmixr2(fit_no_bsv, pred = "IPRED", wres = "IWRES")
  expect_true(xpose::is.xpdb(xpdb))
})

#' Default 'nlmixr2' theme for 'xpose'
#'
#' @description Default 'nlmixr2' theme for 'xpose'.
#'
#' @return A list with 'xpose' theme specifiers.
#'
#' @importFrom ggplot2 labeller label_both
#' @importFrom xpose theme_xp_default
#'
#' @export

theme_xp_nlmixr2 <- function() {
  structure(
    list(
      rounding = 1,
      title_suffix = "",
      subtitle_suffix = "",
      caption_suffix = "",
      facets = NULL,
      ncol = NULL,
      nrow = NULL,
      page = NULL,
      scales = "free",
      shrink = TRUE,
      labeller = ggplot2::labeller(
        .default = ggplot2::label_both,
        .multi_line = FALSE
      ),
      as.table = TRUE,
      switch = NULL,
      drop = TRUE,
      dir = "h",
      strip.position = "top",
      margins = FALSE,
      space = "fixed",
      byrow = TRUE,
      guide_alpha = NA,
      guide_color = "black",
      guide_linetype = "solid",
      guide_size = 0.5,
      line_alpha = 0.5,
      line_color = "#1F4E79",
      line_linetype = "solid",
      line_size = 0.5,
      point_alpha = 0.5,
      point_color = "#1F4E79",
      point_fill = NA,
      point_shape = 16,
      point_size = 2,
      point_stroke = 0.5,
      smooth_alpha = NA,
      smooth_color = "#C00000",
      smooth_fill = "#C00000",
      smooth_linetype = 1,
      smooth_method = "loess",
      smooth_se = FALSE,
      smooth_size = 1,
      smooth_weight = 1,
      text_alpha = NA,
      text_angle = 0,
      text_color = "black",
      text_family = "",
      text_fontface = "plain",
      text_lineheight = 1.2,
      text_size = 3.1,
      text_hjust = 0.5,
      text_vjust = 0.5,
      density_alpha = NA,
      density_color = "black",
      density_fill = NA,
      density_weight = 1,
      density_size = 0.5,
      density_linetype = 2,
      histogram_alpha = NA,
      histogram_color = "black",
      histogram_fill = "#1F4E79",
      histogram_size = 0.5,
      histogram_bins = 10,
      rug_alpha = NA,
      rug_color = "black",
      rug_linetype = 1,
      rug_size = 0.3,
      area_alpha = NA,
      area_color = NA,
      area_fill = "grey35",
      area_size = 0.5,
      area_linetype = 1
    ),
    class = c("xpose_theme", "uneval")
  )
}

#' Data summary function
#'
#' @description Convert 'nlmixr2' model output into an 'xpose' database
#'
#' @param obj nlmixr2 fit object to be evaluated
#' @param model Model. Can be blank
#' @param software Software that generated the model fit
#' @param rounding Number of figures to round estimates to
#' @param runname Name of the model object being converted
#'
#' @return A summary data object used by \link{xpose_data_nlmixr2}.
#'
#' @importFrom magrittr %>%
#' @importFrom dplyr bind_rows filter mutate arrange_at case_when select one_of tibble
#' @importFrom tidyr complete
#' @importFrom rlang syms
#' @importFrom methods is
summarise_nlmixr2_model <- function(obj, model, software, rounding, runname) {
  sum <- dplyr::bind_rows(
    sum_nlmixr2_software(software), # Software name
    sum_nlmixr2_version(software), # Software version
    sum_nlmixr2_file(runname, software), # Model file
    sum_nlmixr2_run(obj, runname, software), # Model run
    sum_nlmixr2_directory(obj, software), # Model directory
    sum_nlmixr2_reference(model, software), # Reference model
    sum_nlmixr2_timestart(obj, model, software), # Run start time
    sum_nlmixr2_timestop(obj, model, software), # Run stop time
    sum_nlmixr2_probn(model, software), # Problem no.
    sum_nlmixr2_label(model, software), # Model label
    sum_nlmixr2_description(model, software), # Model description
    sum_nlmixr2_input_data(obj, model, software), # Model input data used
    sum_nlmixr2_nind(model, software, obj), # Number of individuals
    sum_nlmixr2_nobs(model, software, obj), # Number of observations
    sum_nlmixr2_subroutine(model, software), # Des solver
    sum_nlmixr2_runtime(model, software, obj, rounding), # Estimation runtime
    sum_nlmixr2_simseed(obj, model, software), # Simulation seed
    sum_nlmixr2_covtime(model, software, obj, rounding), # Covariance matrix runtime
    sum_nlmixr2_term(model, software), # Run termination message
    sum_nlmixr2_warnings(model, software), # Run warnings (e.g. boundary)
    sum_nlmixr2_errors(model, software), # Run errors (e.g termination error)
    sum_nlmixr2_nsig(model, software), # Number of significant digits
    sum_nlmixr2_nsim(model, software), # Number of simulations
    sum_nlmixr2_condn(model, software, rounding), # Condition number
    sum_nlmixr2_nesample(model, software), # Number of esample
    sum_nlmixr2_esampleseed(model, software), # esample seed number
    sum_nlmixr2_ofv(model, software, obj, rounding), # Objective function value
    sum_nlmixr2_method(model, software, obj), # Estimation method or sim
    sum_nlmixr2_shk(model, software, 'eps', obj, rounding), # Epsilon shrinkage
    sum_nlmixr2_shk(model, software, 'eta', obj, rounding) # Eta shrinkage
  )

  . = NULL
  problem <- label <- NULL

  # Complete missing cases for consistency
  tmp <- sum %>%
    dplyr::filter(.$problem != 0)

  if (nrow(tmp) == 0) {
    return(sum)
  }

  tmp %>%
    tidyr::complete(
      !!!rlang::syms(c('problem', 'label')),
      fill = list(subprob = 0, value = 'na')
    ) %>%
    dplyr::bind_rows(dplyr::filter(sum, sum$problem == 0)) %>%
    dplyr::arrange_at(.vars = c('problem', 'label', 'subprob')) %>%
    dplyr::mutate(
      descr = dplyr::case_when(
        .$label == 'software' ~ 'Software',
        .$label == 'version' ~ 'Software version',
        .$label == 'file' ~ 'Run file',
        .$label == 'run' ~ 'Run number',
        .$label == 'dir' ~ 'Run directory',
        .$label == 'ref' ~ 'Reference model',
        .$label == 'probn' ~ 'Problem number',
        .$label == 'timestart' ~ 'Run start time',
        .$label == 'timestop' ~ 'Run stop time',
        .$label == 'descr' ~ 'Run description',
        .$label == 'label' ~ 'Run label',
        .$label == 'data' ~ 'Input data',
        .$label == 'nobs' ~ 'Number of observations',
        .$label == 'nind' ~ 'Number of individuals',
        .$label == 'nsim' ~ 'Number of simulations',
        .$label == 'simseed' ~ 'Simulation seed',
        .$label == 'subroutine' ~ 'ADVAN',
        .$label == 'runtime' ~ 'Estimation runtime',
        .$label == 'covtime' ~ 'Covariance step runtime',
        .$label == 'term' ~ 'Termination message',
        .$label == 'warnings' ~ 'Run warnings',
        .$label == 'errors' ~ 'Run errors',
        .$label == 'nsig' ~ 'Number of significant digits',
        .$label == 'condn' ~ 'Condition number',
        .$label == 'nesample' ~ 'Number of ESAMPLE',
        .$label == 'esampleseed' ~ 'ESAMPLE seed number',
        .$label == 'ofv' ~ 'Objective function value',
        .$label == 'method' ~ 'Estimation method',
        .$label == 'epsshk' ~ 'Epsilon shrinkage',
        .$label == 'etashk' ~ 'Eta shrinkage'
      )
    ) %>%
    dplyr::select(dplyr::one_of(
      'problem',
      'subprob',
      'descr',
      'label',
      'value'
    ))
}

# Default template for function output
sum_tpl <- function(label, value) {
  dplyr::tibble(problem = 0, subprob = 0, label = label, value = value)
}

# Software name
sum_nlmixr2_software <- function(software) {
  sum_tpl('software', software)
}

# Software version
sum_nlmixr2_version <- function(software) {
  sum_tpl('version', as.character(utils::packageVersion('nlmixr2est')))
}

# Model object name
sum_nlmixr2_file <- function(runname, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'file', value = runname)
  }
}

# Model run name
sum_nlmixr2_run <- function(obj, runname, software) {
  if (software == 'nlmixr2') {
    if (!is.null(obj$model.name)) {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'run',
        value = obj$model.name
      )
    } else {
      dplyr::tibble(problem = 1, subprob = 0, label = 'run', value = runname)
    }
  }
}

# Model file directory
sum_nlmixr2_directory <- function(obj, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'dir', value = getwd())
  }
}

# Reference model
sum_nlmixr2_reference <- function(model, software) {
  if (software == 'nlmixr2') {
    sum_tpl('ref', 'not implemented')
  }
}

# Run start time
sum_nlmixr2_timestart <- function(obj, model, software) {
  if (software == 'nlmixr2') {
    st <- "not implemented"
    if (!is.null(obj$start.time)) {
      st <- as.character(obj$start.time)
    }
    dplyr::tibble(problem = 0, subprob = 0, label = 'timestart', value = st)
  }
}

# Run stop time
sum_nlmixr2_timestop <- function(obj, model, software) {
  if (software == 'nlmixr2') {
    st <- "not implemented"
    if (!is.null(obj$stop.time)) {
      st <- as.character(obj$stop.time)
    }
    dplyr::tibble(problem = 0, subprob = 0, label = 'timestop', value = st)
  }
}

# Problem no.
sum_nlmixr2_probn <- function(model, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'probn', value = '1')
  }
}

# Model Label
sum_nlmixr2_label <- function(model, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'label', value = 'na')
  }
}

# Model description
sum_nlmixr2_description <- function(model, software) {
  if (software == 'nlmixr2') {
    sum_tpl('descr', 'not implemented')
  }
}

# Input data
sum_nlmixr2_input_data <- function(obj, model, software) {
  if (software == 'nlmixr2') {
    if (!is.null(obj$data.name)) {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'data',
        value = obj$data.name
      )
    } else {
      sum_tpl('data', 'not available')
    }
  }
}

# Number of observations
sum_nlmixr2_nobs <- function(model, software, obj) {
  if (software == 'nlmixr2') {
    dplyr::tibble(
      problem = 1,
      subprob = 0,
      label = 'nobs',
      value = as.character(obj$nobs)
    )
  }
}

# Number of individuals
sum_nlmixr2_nind <- function(model, software, obj) {
  if (software == 'nlmixr2') {
    nind <- obj$nsub
    dplyr::tibble(
      problem = 1,
      subprob = 0,
      label = 'nind',
      value = as.character(nind)
    )
  }
}

# Simulation number
sum_nlmixr2_nsim <- function(model, software) {
  if (software == 'nlmixr2') {
    sum_tpl('nsim', 'not implemented')
  }
}

# Simulation seed
sum_nlmixr2_simseed <- function(obj, model, software) {
  if (software == 'nlmixr2') {
    seed_ch <- "not implemented"
    if (!is.null(obj$seed)) {
      seed_ch <- as.character(obj$seed)
    }
    dplyr::tibble(problem = 1, subprob = 0, label = 'simseed', value = seed_ch)
  }
}

# DES solver
sum_nlmixr2_subroutine <- function(model, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'subroutine', value = 'na')
  }
}

# Estimation runtime
sum_nlmixr2_runtime <- function(model, software, obj, rounding) {
  if (software == 'nlmixr2') {
    rt <- 'na'
    if (is(obj, "nlmixr2FitData")) {
      rt <- sum(as.matrix(obj$time[, names(obj$time) != "covariance"]))
    }
    if (rt != 'na') {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'runtime',
        value = as.character(round(rt, rounding))
      )
    } else {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'runtime',
        value = 'not available'
      )
    }
  }
}

# Covariance matrix runtime
sum_nlmixr2_covtime <- function(model, software, obj, rounding) {
  if (software == 'nlmixr2') {
    rt <- obj$time$covariance
    if (!is.null(rt)) {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'covtime',
        value = as.character(round(rt, rounding))
      )
    } else {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'covtime',
        value = 'not available'
      )
    }
  }
}

# Run termination
sum_nlmixr2_term <- function(model, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'term', value = 'na')
  }
}


# Run warnings (e.g. boundary)
sum_nlmixr2_warnings <- function(model, software) {
  if (software == 'nlmixr2') {
    # Can get warnings in $warnings
    dplyr::tibble(problem = 1, subprob = 0, label = 'warnings', value = 'na')
  }
}

# Run errors (e.g termination error)
sum_nlmixr2_errors <- function(model, software) {
  if (software == 'nlmixr2') {
    sum_tpl('errors', 'na')
  }
}

# Number of significant digits
sum_nlmixr2_nsig <- function(model, software) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'nsig', value = 'na')
  }
}

# Condition number
sum_nlmixr2_condn <- function(model, software, rounding) {
  if (software == 'nlmixr2') {
    dplyr::tibble(
      problem = 1,
      subprob = 0,
      label = 'condn',
      value = 'not implemented'
    )
  }
}

# Number of ESAMPLE (i.e. NPDE)
sum_nlmixr2_nesample <- function(model, software) {
  if (software == 'nlmixr2') {
    sum_tpl('nesample', 'na')
  }
}

# ESAMPLE seed number
sum_nlmixr2_esampleseed <- function(model, software) {
  if (software == 'nlmixr2') {
    sum_tpl('esampleseed', 'na')
  }
}

# Objective function value
sum_nlmixr2_ofv <- function(model, software, obj, rounding) {
  if (software == 'nlmixr2') {
    ofv <- obj$objective
    if (!is.null(ofv)) {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'ofv',
        value = as.character(round(ofv, digits = rounding))
      )
    } else {
      dplyr::tibble(
        problem = 1,
        subprob = 0,
        label = 'ofv',
        value = 'not available'
      )
    }
  }
}

# Estimation method or sim
sum_nlmixr2_method <- function(model, software, obj) {
  if (software == 'nlmixr2') {
    dplyr::tibble(problem = 1, subprob = 0, label = 'method', value = obj$est)
  }
}

# Epsilon/Eta shrinkage
sum_nlmixr2_shk <- function(model, software, type, obj, rounding) {
  if (software == 'nlmixr2') {
    shk <- 'na'
    lab <- paste(type, 'shk', sep = '')
    if (any("nlmixr2FitData" == class(obj))) {
      if (type == "eps") {
        shk <- paste(
          round((1 - stats::sd(obj$IWRES)) * 100, digits = rounding),
          "[1]",
          sep = " "
        )
      }
      if (type == "eta") {
        omega <- diag(obj$omega)
        if (length(omega) > 0) {
          d <- as.data.frame(obj[!duplicated(obj$ID), ])

          ## add ETA if missing
          if (!all(names(omega) %in% names(d))) {
            d <- merge(d, obj$eta)
          }

          ## account for 1-eta systems; drop etas absent from d (e.g. eta ~ 0)
          if (length(names(omega)) == 1) {
            d <- data.frame(eta = d[, names(d) %in% names(omega)])
            names(d) <- names(omega)
          } else {
            d <- d[, names(d) %in% names(omega)]
          }
          omega <- omega[names(omega) %in% names(d)]

          eshr <- c()
          for (i in 1:length(omega)) {
            shr <- (1 - (stats::sd(d[, i]) / sqrt(omega[i]))) * 100
            eshr <- c(eshr, round(shr, 3))
          }
          shk <- paste(
            paste(
              round(eshr, digits = rounding),
              ' [',
              1:length(eshr),
              ']',
              sep = ''
            ),
            collapse = ', '
          )
        }
      }
    }
    dplyr::tibble(problem = 1, subprob = 0, label = lab, value = shk)
  }
}

# no-IIV fit without pred/wres gives a descriptive error (#8)

    Code
      xpose_data_nlmixr2(fit_no_bsv)
    Condition
      Error:
      ! Could not automatically determine the weighted residual column. The fit does not contain CWRES, NPDE, or RES. For models without IIV or using ll(), specify `pred` and `wres` manually (e.g. pred = "IPRED", wres = "IWRES").

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xpose_data_nlmixr.R
\name{xpose_data_nlmixr2}
\alias{xpose_data_nlmixr2}
\alias{xpose_data_nlmixr}
\title{Import nlmixr2 output into xpose object}
\usage{
xpose_data_nlmixr2(
  obj = NULL,
  pred = NULL,
  wres = NULL,
  gg_theme = theme_readable,
  xp_theme = theme_xp_default(),
  quiet,
  skip = NULL,
  ...
)

xpose_data_nlmixr(
  obj = NULL,
  pred = NULL,
  wres = NULL,
  gg_theme = theme_readable,
  xp_theme = theme_xp_default(),
  quiet,
  skip = NULL,
  ...
)
}
\arguments{
\item{obj}{nlmixr2 fit object to be evaluated.}

\item{pred}{Name of the population prediction variable to use for
plotting. If unspecified, it will choose either "NPDE",
"CWRES", and "RES" (in that order) if the column exists in the
data.}

\item{wres}{Name of the weighted residual variable to use for
plotting. If unspecified, it will choose either "NPDE",
"CWRES", and "RES" (in that order) if the column exists in the
data.}

\item{gg_theme}{A ggplot2 theme object.}

\item{xp_theme}{An xpose theme or vector of modifications to the
xpose theme (eg. \code{c(point_color = 'red', line_linetype =
'dashed')}).}

\item{quiet}{Logical, if \code{FALSE} messages are printed to the
console.}

\item{skip}{Character vector be used to skip the import/generation
of: 'data', 'files', 'summary' or any combination of the three.}

\item{...}{Additional arguments to be passed to the
\code{\link[readr]{read_delim}} functions.}
}
\value{
An \code{\link[xpose]{xpose_data}} object suitable for use in 'xpose'.
}
\description{
Convert 'nlmixr2' model output into an 'xpose' database.
}
\examples{
\dontrun{
library(nlmixr2)

one.cmt <- function() {
  ini({
    ## You may label each parameter with a comment
    tka <- 0.45 # Ka
    tcl <- log(c(0, 2.7, 100)) # Log Cl
   ## This works with interactive models
    ## You may also label the preceding line with label("label text")
    tv <- 3.45; label("log V")
    ## the label("Label name") works with all models
    eta.ka ~ 0.6
    eta.cl ~ 0.3
    eta.v ~ 0.1
    add.sd <- 0.7
  })
  model({
    ka <- exp(tka + eta.ka)
    cl <- exp(tcl + eta.cl)
    v <- exp(tv + eta.v)
    linCmt() ~ add(add.sd)
  })
}

theo_sd_fit <- nlmixr2(one.cmt, theo_sd, "focei", control=foceiControl(print=0))

xpdb <- xpose_data_nlmixr2(obj = theo_sd_fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarise_nlmixr_model.R
\name{summarise_nlmixr2_model}
\alias{summarise_nlmixr2_model}
\title{Data summary function}
\usage{
summarise_nlmixr2_model(obj, model, software, rounding, runname)
}
\arguments{
\item{obj}{nlmixr2 fit object to be evaluated}

\item{model}{Model. Can be blank}

\item{software}{Software that generated the model fit}

\item{rounding}{Number of figures to round estimates to}

\item{runname}{Name of the model object being converted}
}
\value{
A summary data object used by \link{xpose_data_nlmixr2}.
}
\description{
Convert 'nlmixr2' model output into an 'xpose' database
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_xp_nlmixr.R
\name{theme_xp_nlmixr2}
\alias{theme_xp_nlmixr2}
\title{Default 'nlmixr2' theme for 'xpose'}
\usage{
theme_xp_nlmixr2()
}
\value{
A list with 'xpose' theme specifiers.
}
\description{
Default 'nlmixr2' theme for 'xpose'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nlmixr_vpc_theme.R
\docType{data}
\name{nlmixr2_vpc_theme}
\alias{nlmixr2_vpc_theme}
\title{Default VPC theme for 'xpose.nlmixr2'}
\format{
An object of class \code{vpc_theme} of length 23.
}
\usage{
nlmixr2_vpc_theme
}
\value{
A list with 'vpc' theme specifiers.
}
\description{
Default VPC theme for 'xpose.nlmixr2'.
}
\keyword{datasets}
